/* Network Monitor Using X Data and Struct Defines File */

#define SNAPLEN MTU		/* Ethernet Packet Length*/
#define NPSNAP 1		/* Number of Packet Snapped*/
#define TELNET_ENTER 0x0d	/* Enter in Telnet*/
#define FTP_ENTER 0x0d		/* Enter in Ftp*/

#define ETHERHEAD 14		/*Ethernet Packet Head Length */
#define PPPHEAD    4		/* PPP Packet Header Length */
#define SLIPHEAD  16		/* SLIP Packet Header Length */

#define IP_VERSION 4
#define URG 32			/* TCP-flags */
#define ACK 16
#define PSH  8
#define RST  4
#define SYN  2
#define FIN  1

#define NO_IP 0			/* Protocol Numbers */
#define NO_IP_4 1000
#define ICMP       1
#define TCP        6
#define UDP       17

#define ICMP_HEADLENGTH 4	/* Fixed ICMP Header Length */
#define UDP_HEADLENGTH 8	/* Fixed UDP Header Length */

#define IP_DELAY   32
#define IP_THROUGHPUT 16
#define IP_RELIABILITY 8

#define IP_DF  2
#define IP_MF  1

/* ICMP Types */
#define ICMP_TYPE_0     "Echo reply"
#define ICMP_TYPE_3     "Destination unreachable"
#define ICMP_TYPE_4     "Source quench"
#define ICMP_TYPE_5     "Redirect"
#define ICMP_TYPE_8     "Echo"
#define ICMP_TYPE_11    "Time exceeded"
#define ICMP_TYPE_12    "Parameter problem"
#define ICMP_TYPE_13    "Timestamp"
#define ICMP_TYPE_14    "Timestamp reply"
#define ICMP_TYPE_15    "Information request"
#define ICMP_TYPE_16    "Information reply"
#define ICMP_TYPE_17    "Address mask request"
#define ICMP_TYPE_18    "Adress mask reply"


struct packetheader
{
        struct timeval ts;             /* time stamp */
        unsigned long caplen;          /* length of portion present */
        unsigned long len;             /* length this packet (off wire) */
};           

struct IP_header                        /* The IPheader (without options) */
{
	unsigned char verlen, type;
	unsigned short length, ID, flag_offset;
	unsigned char TTL, protocol;
	unsigned short checksum;
	unsigned long int source, destination;
};

struct pseudo_IP_header 
{
        unsigned long int source, destination;
        char zero_byte, protocol;
        unsigned short TCP_UDP_len;
};            

struct TCP_header                       /* The TCP header (without options) */
{
	unsigned short source, destination;
	unsigned long int seq_nr, ACK_nr;
	unsigned short offset_flag, window, checksum, urgent; 
};

struct ICMP_header                                /* The ICMP header */ 
{
	unsigned char type, code;
	unsigned short checksum; 
};

struct UDP_header                                /* The UDP header */ 
{
	unsigned short source, destination;
	unsigned short length, checksum;
};

struct unwrap                                           /* some extra info */
{
	int IP_len, TCP_len, ICMP_len, UDP_len;         /* header lengths */ 
	int DATA_len;
	int TOTAL_len;
};

struct con
{
	unsigned long int ips,ipc;		/* The Two IP Address */
	unsigned int ports,portc;		/* The Two Port Number */
	int protocol;				/* The Protocol Number */
        time_t last_packet;
        char  csbuf[80];
        char  scbuf[2048];
        short csbegin;
        short csend;
        short scbegin;
        short scend;
        int   bytes_sent;
        int   last_bytes_sent;
	/* Only reason I used this is that I have no much idea of xview program */
	int bydirection;
	int   curlog;/* 0=none,1=raw,2=text ,only used when xv==1 */
/*
 *	Log have been changed to have a wider meaning:
 *  	log = -2: Not log, and not show in rowlist.
 *		Process in pcap filter.
 *	log = -1: Show in rowlist, when clicked show connections content.
 *		Process but will not exec circle_copy.
 *	log =  0: Show in rowlist, log content in csbuf and scbuf.
 *		Process normaly.
 *	log =  1: Show in rowlist, log content in file(RAW).
 *		Process log, but will not exec circle_copy.
 *	log =  2: Show in rowlist, log content in file(TEXT).
 *		Process log, but will not exec circle_copy.
 *	log =  3: Show in rowlist, log content both in csbuf,scbuf and file(RAW).
 *		Process both (avoid use this).
 *	log =  4: Show in rowlist, log content both in csbuf,scbuf and file(TEXT).
 *		Process both (avoid use this).
 *	Default is -1. (TO AVOID MUCH PROBLEMS IN PROCESS)
*/
        char  log;
        char  logfile[60];
        int   logfd;

	/* pconrow is the corrlate con_row .  */
	struct con_row *pconrow;
	struct log_log *plogl;
	struct  con *next;
};

struct con_row
{
	char ipaddress_s[50],ipaddress_c[50];	/* String of IP Address */
	char proto[50];				/* String of Protocol */
	char name[80];
	int changed;
	/*int  numrow;*/
/* The pcon is the correlate con */
	struct con * pcon;
	struct con_row *prev;
	struct con_row *next;
};

struct log_packet
{
	time_t time;
	int data_len;
};

struct Service
{
	int pnum;
	char * ser;
	struct Service *next;
};

struct log_log
{
	/* Ttyfd is the fd of term-emulator.   */
	struct con *pcon;
	int   ttyfd;
	pid_t pid_term;
	struct log_log *next;
};
	
#define WHITE   0
#define RED     1
#define GREEN   2
#define BLUE    3
#define BLACK   4
#define NUM_COLORS      5

#define SHMSIZE 1024
#define HASHSIZE 1024
#define ANDMASK 1023
#define DEBUG_DEVICE "./debuglog"

/* Used to log into TERM_EMULATOR */
#define LOG_MAX 10
#define TRY_MAX 10
#define SHMKEY ((key_t) 7890)
#define SEKEY1 ((key_t) 7891) /* Client semaphore key */
#define SEKEY2 ((key_t) 7892) /* Server semaphore key */
#define PERMS 0666

struct Log_init
{
	unsigned long int ips_b,ips_e,ipd_b,ipd_e;
	unsigned int ports_b,ports_e,portd_b,portd_e;
	int logstat;
	struct Log_init *next;
};

#ifndef HAVE_SEMUN_UNION
union semun
{                           /* define union for semctl() */
	int val;
	struct semid_ds *buf;
	unsigned short  *array;
};
#endif

